@@ -1,5 +1,5 @@
 /*
- * $Id: dbd_dbcheck.c,v 1.1.2.2 2005/01/03 13:49:56 didg Exp $
+ * $Id: dbd_dbcheck.c,v 1.4 2009-05-06 11:54:24 franklahm Exp $
  *
  * Copyright (C) Joerg Lenneis 2003
  * All Rights Reserved.  See COPYING.
@@ -22,22 +22,16 @@
 #include "dbif.h"
 #include "dbd.h"
 
-int dbd_check(char *dbdir)
+int dbd_check_indexes(DBD *dbd, char *dbdir)
 {
     u_int32_t c_didname = 0, c_devino = 0, c_cnid = 0;
-#if 0
-    char dbdir[MAXPATHLEN];
 
-    if (NULL == getcwd(dbdir, sizeof(dbdir)) )
-        return -1;
-#endif
+    LOG(log_note, logtype_cnid, "CNID database at `%s' is being checked (quick)", dbdir);
 
-    LOG(log_debug, logtype_cnid, "CNID database at `%s' is being checked (quick)", dbdir);
-
-    if (dbif_count(DBIF_IDX_CNID, &c_cnid)) 
+    if (dbif_count(dbd, DBIF_CNID, &c_cnid)) 
         return -1;
 
-    if (dbif_count(DBIF_IDX_DEVINO, &c_devino))
+    if (dbif_count(dbd, DBIF_IDX_DEVINO, &c_devino))
         return -1;
 
     /* bailout after the first error */
@@ -46,7 +40,7 @@ int dbd_check(char *dbdir)
         return 1;
     }
 
-    if (dbif_count(DBIF_IDX_DIDNAME, &c_didname)) 
+    if (dbif_count(dbd, DBIF_IDX_DIDNAME, &c_didname)) 
         return -1;
     
     if ( c_cnid != c_didname) {
@@ -54,7 +48,8 @@ int dbd_check(char *dbdir)
         return 1;
     }
 
-    LOG(log_debug, logtype_cnid, "CNID database at `%s' seems ok, %u entries.", dbdir, c_cnid);
+    LOG(log_note, logtype_cnid, "CNID database at `%s' seems ok, %u entries.", dbdir, c_cnid);
     return 0;  
 }
 
+
