@@ -1,26 +1,15 @@
 /*
+ * $Id: at_proto.c,v 1.2 2001-06-29 14:14:47 rufustfirefly Exp $
+ *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
- * All Rights Reserved.
  *
- * Permission to use, copy, modify, and distribute this software and
- * its documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appears in all copies and
- * that both that copyright notice and this permission notice appear
- * in supporting documentation, and that the name of The University
- * of Michigan not be used in advertising or publicity pertaining to
- * distribution of the software without specific, written prior
- * permission. This software is supplied as is without expressed or
- * implied warranties of any kind.
- *
- *	Research Systems Unix Group
- *	The University of Michigan
- *	c/o Mike Clark
- *	535 W. William Street
- *	Ann Arbor, Michigan
- *	+1-313-763-0525
- *	netatalk@itd.umich.edu
+ * All Rights Reserved.  See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/protosw.h>
 #include <sys/domain.h>
 #include <sys/types.h>
@@ -36,7 +25,7 @@ extern int		ddp_init();
 extern int		ddp_ifoutput();
 extern int		ddp_ifinput();
 extern int		ddp_ifioctl();
-#endif ultrix
+#endif /* ultrix */
 
 struct protosw		atalksw[] = {
     {
@@ -58,7 +47,7 @@ struct protosw		atalksw[] = {
 #ifdef ultrix
 	/* interface hooks */
 	ddp_ifoutput,	ddp_ifinput,	ddp_ifioctl,	0,
-#endif ultrix
+#endif /* ultrix */
     },
 };
 
